;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Teaser 2                                                      ;
;                                                               ;
; Copyright (c)2012 James Vernon                                ;
; jamesv82@live.com.au                                          ;
; http://www.jvti.org                                           ;
;                                                               ;
; You are allowed to use parts of this code as long as you give ;
; credit where it is due :)                                     ;
;                                                               ;
;===============================================================;

#include "ab5t2.h"                              ; header file

;------------------------------------------------
; Start of program
;------------------------------------------------
start:
        set     textwrite,(iy+sgrflags)         ; write text to gbuf

        ld      b,50
        call    wait

        ld      hl,strJVTI
        call    displayPage                     ; "JVTI presents..."
        ld      b,50
        call    wait                            ; wait for a bit

        call    blackenScreen                   ; blacken the screen

        ld      hl,strScreen1                   ; HL => first screen string data
        call    displayStrings                  ; display them
        call    invertScreen                    ; invert screen colours
        call    ionFastCopy
        ld      b,150
        call    wait

        ld      hl,strScreen2                   ; HL => second screen string data
        call    displayStrings                  ; display them
        call    invertScreen                    ; invert screen colours
        call    ionFastCopy
        ld      b,150
        call    wait

        bcall(_grbufclr)
        call    ionFastCopy                     ; clear screen
        ld      b,20
        call    wait

        ld      hl,strIntex
        call    displayPage                     ; "INTEX SECURITY" etc
        ld      b,120
        call    wait

        bcall(_grbufclr)

        call    invertScreen                    ; black
        call    ionFastCopy
        ld      b,8
        call    wait

        call    invertScreen                    ; white
        call    ionFastCopy
        ld      b,8
        call    wait

        call    invertScreen                    ; black
        call    ionFastCopy
        ld      b,8
        call    wait

; scroll splash up
        ld      de,gbuf+768-12                  ; DE => last line of gbuf
        ld      bc,12                           ; BC = 12 bytes to copy for first scroll loop
        ld      a,64                            ; A = 64 lines to scroll
scrollSplashLoop:
        push    af                              ; save line counter
        push    de                              ; save gbuf ptr
        push    bc                              ; save length value
        ld      hl,splashData                   ; HL => start of splash screen data
        ldir                                    ; load BC bytes of splash data to (DE) on gbuf
        call    ionFastCopy                     ; update gbuf to lcd
        pop     bc
        ld      hl,12
        add     hl,bc
        ld      b,h
        ld      c,l                             ; BC = number of bytes to copy next loop
        pop     de
        ld      hl,-12
        add     hl,de
        ex      de,hl                           ; DE => previous line on gbuf
        pop     af
        dec     a
        jr      nz,scrollSplashLoop             ; repeat A times
        ld      b,50
        call    wait

; split screen down the middle and scroll each half to left & right side, revealing white screen
        ld      b,48                            ; B = 48 pixels to scroll each half
splitSplashLoop:
        push    bc                              ; save main counter
        ld      hl,gbuf                         ; HL => start of gbuf
        ld      b,64                            ; B = 64 lines to scroll
splitSplashScrollLoop:
        ld      de,5
        add     hl,de                           ; HL => 6th byte on line
        or      a                               ; clear carry
        rl      (hl)                            ; rotate left
        dec     hl                              ; update ptr
        rl      (hl)                            ; 6 times to scroll the left half 1 pixel to the left
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        ld      de,6
        add     hl,de                           ; HL => 7 byte on line
        or      a                               ; clear carry
        rr      (hl)                            ; rotate right
        inc     hl                              ; update ptr
        rr      (hl)                            ; 6 times to scroll the right half 1 pixel to the right
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl                              ; HL => next line on gbuf
        djnz    splitSplashScrollLoop           ; repeat for all 64 lines
        call    ionFastCopy                     ; update lcd to show progress
        pop     bc                              ; restore main counter
        djnz    splitSplashLoop

        ld      b,10
        call    wait

        ld      hl,logoData
        ld      de,gbuf+(4*12)
        ld      bc,12*10
        ldir
        call    ionFastCopy                     ; show AB5 logo
        ld      b,20
        call    wait

        ld      hl,strRebirth
        call    displayPageLoop                 ; "The rebirth of the war..." (skipping the gbuf clear)
        ld      b,20
        call    wait                            ; wait for a bit

        ld      hl,strCredits                   ; HL => credits string data
        call    displayStringsAfterClr          ; display them (without clearing gbuf)
        call    ionFastCopy
        call    waitKey

quit:
        res     textwrite,(iy+sgrflags)         ; write text directly to lcd
        ret

;------------------------------------------------
; waitKey - wait for a key press
;
; Input:    none
; Output:   A = key pressed (_getcsc style)
;------------------------------------------------
waitKey:
        ei                                      ; enable interrupts
        res     apdrunning,(iy+apdflags)        ; disable apd
wKeyLoop:
        halt
        bcall(_getcsc)                          ; A = key pressed
        or      a                               ; check if 0
        jr      z,wKeyLoop                      ; if so, no key was pressed
        set     apdrunning,(iy+apdflags)        ; enable apd
        ret

;------------------------------------------------
; displayChar - display a character to gbuf and update pencol & penrow accordingly
;
; input:    HL => character to display
; output:   HL => next character
;------------------------------------------------
displayChar:
        ld      a,(hl)                          ; A = character
        inc     hl                              ; HL => next character
        push    hl                              ; save character pointer
        bcall(_vputmap)                         ; display character and update pencol & penrow accordingly
        pop     hl                              ; restore character pointer
        ret

;------------------------------------------------
; displayPage - display a page of text using the displayChar routine and update it to the lcd
;
; input:    HL => page string data, each line NULL terminated, page is terminated by $FF
; output:   HL => data directly after this page's data
;------------------------------------------------
displayPage:
        push    hl                              ; save data ptr
        bcall(_grbufclr)
        pop     hl                              ; restore data ptr
displayPageLoop:
        ld      d,(hl)                          ; D = new penrow
        inc     hl
        ld      e,(hl)                          ; E = new pencol
        inc     hl                              ; HL => next line string
        ld      (pencol),de                     ; set cursor for new line
displayLineLoop:
        ld      a,(hl)                          ; A = character
        inc     a                               ; was it $FF
        jr      z,displayPageEndPage            ; if so, update ptr to next page data and ret
        dec     a                               ; was it $00
        jr      z,displayPageEndLine
        call    displayChar                     ; display char to gbuf
        push    hl                              ; save char ptr
        call    ionFastCopy                     ; update lcd
        ld      b,2
        call    wait                            ; double halt B times
        pop     hl                              ; restore char ptr
        jr      displayLineLoop

displayPageEndLine:
        inc     hl                              ; HL => next line data
        jr      displayPageLoop                 ; load new pen coords & repeat
displayPageEndPage:
        inc     hl                              ; HL => data directly after this page's data
        ret

;------------------------------------------------
; invertScreen - invert gbuf and display it
;
; Input:    none
; Output:   none
;------------------------------------------------
invertScreen:
        ld      bc,64*12                        ; BC = 64 rows of 12 bytes to invert
        ld      hl,gbuf                         ; HL => start of gbuf
invertScreenLoop:
        ld      a,(hl)
        cpl                                     ; invert byte
        ld      (hl),a                          ; and put it back to gbuf
        inc     hl                              ; inc gbuf ptr
        dec     bc                              ; dec counter
        ld      a,b
        or      c                               ; does BC = 0?
        jr      nz,invertScreenLoop
        jp      ionFastCopy                     ; show gbuf and ret

;------------------------------------------------
; displayStrings - display a series of strings
;
; input:    HL => start of string data
; output:   HL => after string data
;------------------------------------------------
displayStrings:
        push    hl                              ; save ptr
        bcall(_grbufclr)
        pop     hl                              ; restore ptr
displayStringsAfterClr:
        ld      b,(hl)                          ; B = number of strings
        inc     hl                              ; HL => first string, starting with pen coords
displayStringsLoop:
        push    bc                              ; save counter
        ld      d,(hl)                          ; D = penrow
        inc     hl
        ld      e,(hl)                          ; E = pencol
        ld      (pencol),de                     ; set it
        inc     hl                              ; HL => string
        bcall(_vputs)                           ; display string; HL => next string pen coords
        pop     bc                              ; restore counter
        djnz    displayStringsLoop
        ret

;------------------------------------------------
; wait - wait for a specified time
;
; Input:    B = number of times to double halt
; Output:   none
;------------------------------------------------
wait:
        ei
waitLoop:
        halt
        halt
        halt
        halt
        djnz    waitLoop
        ret

#include "blacken.asm"                          ; screen blacken routine

;------------------------------------------------
; Start of data
;------------------------------------------------
strJVTI:
.db     28,22,"JVTI presents...",$FF

strScreen1:
.db     3
.db     20,0,"In the cold depths of space,",0
.db     27,2,"horrors exist that mankind",0
.db     34,3,"never could have foreseen..",0

strScreen2:
.db     3
.db     14,2,"For 10 years, we thought",0
.db     21,2,"they were gone...",0
.db     44,36,"We were wrong...",0

strIntex:
.db     7,1,"INTEX SECURITY SYSTEM",0
.db     14,1,"REPORTS UNIDENTIFIED",0
.db     21,1,"MOVEMENT IN ENGINEERING",0
.db     35,1,"PROCEED WITH CAUTION...",$FF

strRebirth:
.db     14,9,"The rebirth of the war...",$FF

strCredits:
.db     5
.db     25,14,"Based on the original",0
.db     31,13,"ALIEN BREED series by",0
.db     37,35,"TEAM 17",0
.db     47,21,"by James Vernon",0
.db     54,26,"WWW.JVTI.ORG",0

logoData:
.db     $00,$03,$d8,$6f,$fc,$1f,$3e,$7f,$ff,$83,$f0,$00
.db     $00,$07,$d8,$6f,$fe,$1f,$bf,$7f,$ff,$c3,$f0,$00
.db     $00,$0c,$d8,$60,$07,$00,$00,$00,$00,$e0,$00,$00
.db     $00,$0f,$d8,$6f,$33,$1f,$3e,$79,$e6,$63,$e0,$00
.db     $00,$0f,$d8,$6f,$33,$1f,$bf,$79,$e6,$63,$f0,$00
.db     $00,$0c,$d8,$6c,$33,$19,$b3,$61,$86,$60,$30,$00
.db     $00,$0c,$dc,$6c,$33,$19,$b3,$61,$86,$60,$30,$00
.db     $00,$0c,$df,$6f,$b3,$1f,$b3,$7d,$f7,$e3,$f0,$00
.db     $00,$0c,$cf,$67,$b3,$1f,$33,$3c,$f7,$e3,$e0,$00
.db     $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

splashData:
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$f0,$3f,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$e0,$1f,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$60,$1b,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$c0,$0f,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fc,$0c,$c0,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fc,$3c,$f0,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$f0,$1c,$e0,$3f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$e0,$00,$00,$1f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$80,$00,$00,$07,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$f0,$00,$00,$3f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$f0,$00,$00,$3f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$f0,$00,$00,$3f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$e0,$00,$00,$1f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$c0,$f0,$3c,$0f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$b3,$0f,$c3,$37,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$f3,$00,$03,$3f,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fb,$00,$0b,$7f,$ff,$ff,$ff,$ff
 .db $ff,$f9,$bd,$0b,$7b,$02,$03,$78,$c6,$10,$8f,$ff
 .db $ff,$f6,$bd,$79,$7b,$00,$03,$7b,$5a,$f7,$b7,$ff
 .db $ff,$f6,$bd,$7a,$7b,$10,$23,$7b,$5a,$f7,$b7,$ff
 .db $ff,$f0,$bd,$1b,$7d,$10,$02,$f8,$c6,$31,$b7,$ff
 .db $ff,$f6,$bd,$7b,$7d,$00,$02,$fb,$5a,$f7,$b7,$ff
 .db $ff,$f6,$bd,$7b,$7d,$00,$02,$fb,$5a,$f7,$b7,$ff
 .db $ff,$f6,$bd,$7b,$7d,$60,$1a,$fb,$5a,$f7,$b7,$ff
 .db $ff,$f6,$bd,$7b,$7d,$7d,$fa,$fb,$5a,$f7,$b7,$ff
 .db $ff,$f6,$bd,$7b,$7d,$7d,$fa,$fb,$5a,$f7,$b7,$ff
 .db $ff,$f6,$bd,$7b,$7d,$7d,$fa,$fb,$5a,$f7,$b7,$ff
 .db $ff,$f6,$85,$0b,$7d,$7f,$fa,$f8,$da,$10,$8f,$ff
 .db $ff,$ff,$ff,$ff,$fd,$fd,$fe,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fd,$ff,$fe,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fd,$fd,$fa,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fd,$7d,$fa,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fd,$7d,$fa,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fd,$3d,$f2,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fd,$00,$02,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fe,$80,$45,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fc,$88,$04,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fd,$c8,$0e,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fe,$c1,$0d,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$fe,$64,$1d,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$3f,$f3,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$80,$07,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$c0,$0f,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$e0,$1f,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
 .db $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

.end
